
#include "glbox.h"
#include "evalutor.h"

#if defined(_CC_MSVC_)
#pragma warning(disable:4305)
#endif

GLBox::GLBox( QWidget* parent, const char* name, Evalutor* eval )
    : QGLWidget( parent, name )
{
	// rotace
    xRot = yRot = zRot = 0.0;
	
	// meritko sceny
    scale = 1.25;
	
	// jina inicializace
    object = NULL;

	domain_x_from = domain_y_from = -15.0;
	domain_x_to = domain_y_to = 15.0;
	domain_x_step = domain_y_step = 0.25;

	this->eval = eval;
}


GLBox::~GLBox()
{
    glDeleteLists( object, 1 );
}

void GLBox::paintGL()
{
    glClear( GL_COLOR_BUFFER_BIT );

    glLoadIdentity();
    glTranslatef( 0.0, 0.0, zPosition );
    glScalef( scale, scale, scale );

    glRotatef( xRot, 1.0, 0.0, 0.0 ); 
    glRotatef( yRot, 0.0, 1.0, 0.0 ); 
    glRotatef( zRot, 0.0, 0.0, 1.0 );

    glCallList( object );
}

void GLBox::initializeGL()
{
	glEnable (GL_BLEND | GL_LINE_SMOOTH | GL_DEPTH_TEST);
	glBlendFunc (GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	glHint (GL_LINE_SMOOTH_HINT, GL_DONT_CARE);
	qglClearColor( black );

	// vytvorim prvni seznam objektu
    object = makeObject();
}

void GLBox::resizeGL( int w, int h )
{
    glViewport( 0, 0, (GLint)w, (GLint)h );
    glMatrixMode( GL_PROJECTION );
    glLoadIdentity();
	gluPerspective(50, (GLfloat) w / (GLfloat) h, 3.0, 150.0);
    //glFrustum( -1.0, 1.0, -1.0, 1.0, 3.0f, 150.0f );
    glMatrixMode( GL_MODELVIEW );
}

GLuint GLBox::makeObject()
{	
    GLuint list;

    list = glGenLists( 1 );

    glNewList( list, GL_COMPILE );

    glLineWidth( 2.0 );

	// OSY
	glColor3f(1.0, 0.0, 0.0);
	glBegin(GL_LINES);
	glVertex3f(-500.0f, 0.0, 0.0);
	glVertex3f(500.0f, 0.0, 0.0);
	glEnd();

	glColor3f(0.0, 1.0, 0.0);
	glBegin(GL_LINES);
	glVertex3f(0.0, 500.0f, 0.0);
	glVertex3f(0.0, -500.0f, 0.0);
	glEnd();

	glColor3f(1.0, 1.0, 0.0);
	glBegin(GL_LINES);
	glVertex3f(0.0, 0.0, -500.0f);
	glVertex3f(0.0, 0.0, 500.0f);
	glEnd();

	// OBJEKTY

    qglColor( white );

    glLineWidth( 1.0 );

	// cary rovnobezne s X
	for (double x = domain_x_from; x <= domain_x_to; x += domain_x_step) {
	    glBegin( GL_LINE_STRIP );
		for (double y = domain_y_from; y <= domain_y_to; y += domain_y_step) {
			glVertex3f(  x,  y, eval->TheFunction(x, y));
		}
		glEnd();
	}

	// cary rovnobezne s Y
	for (double y = domain_y_from; y <= domain_y_to; y += domain_y_step) {
	    glBegin( GL_LINE_STRIP );
		for (double x = domain_x_from; x <= domain_x_to; x += domain_x_step) {
			glVertex3f(  x,  y, eval->TheFunction(x, y));
		}
		glEnd();
	}

    glEndList();

    return list;
}

void GLBox::setXRotation( int degrees )
{
    xRot = (GLfloat)(degrees % 360);
    updateGL();
}

void GLBox::setYRotation( int degrees )
{
    yRot = (GLfloat)(degrees % 360);
    updateGL();
}

void GLBox::setZRotation( int degrees )
{
    zRot = (GLfloat)(degrees % 360);
    updateGL();
}

void GLBox::setXRotationFactor( int degrees )
{
    xRotFact = degrees;
}

void GLBox::setYRotationFactor( int degrees )
{
    yRotFact = degrees;
}

void GLBox::setZRotationFactor( int degrees )
{
    zRotFact = degrees;
}

void GLBox::setZPosition( int x )
{
    zPosition = (float) x / 10;
	qWarning("%f\n", zPosition);
    updateGL();
}


void GLBox::RebuildScene()
{
	if (object != NULL)
	    glDeleteLists( object, 1 );

	// vytvorim seznam objektu
    object = makeObject();
    updateGL();
}

void GLBox::timerMove()
{
    xRot += xRotFact;
    yRot += yRotFact;
    zRot += zRotFact;
//	if (xRot > 180.0f) xRot = 0;
    updateGL();
}
