
#include <qpushbutton.h>
#include <qslider.h>
#include <qlayout.h>
#include <qframe.h>
#include <qmenubar.h>
#include <qpopupmenu.h>
#include <qapplication.h>
#include <qkeycode.h>
#include <qinputdialog.h>
#include <qmessagebox.h>
#include <qtimer.h>
#include "globjwin.h"
#include "glbox.h"
#include "evalutor.h"
#include "dlgFunction.h"

GLObjectWindow::GLObjectWindow( QWidget* parent, const char* name )
    : QWidget( parent, name )
{
	// Init the object
	rotating = false;
	eval = new Evalutor();
	dlg = new dialogFunctionImpl(eval, this, "Function", TRUE, 0);

    // Create a menu
    QPopupMenu *file = new QPopupMenu( this );
    file->insertItem( "O progr&amu",  this, SLOT(menuAbout()), CTRL+Key_A );
    file->insertItem( "Kon&ec",  qApp, SLOT(quit()), CTRL+Key_E );

    QPopupMenu *rotation = new QPopupMenu( this );
    rotation->insertItem( "&Start rotace",  this, SLOT(menuRotateStart()), CTRL+Key_R );
    rotation->insertItem( "&Konec rotace",  this, SLOT(menuRotateStop()), CTRL+Key_S );
    rotation->insertItem( "&Faktor rotace",  this, SLOT(menuRotateFactors()), CTRL+Key_F );

    QPopupMenu *function = new QPopupMenu( this );
    function->insertItem( "&Definuj", this, SLOT(menuFunctionDefine()), CTRL+Key_D );
    function->insertItem( "&Vykresli", this, SLOT(menuFunctionPlot()), CTRL+Key_V );

    // Create a menu bar
    QMenuBar *m = new QMenuBar( this );
    m->setSeparator( QMenuBar::InWindowsStyle );
    m->insertItem("&Soubor", file );
    m->insertItem("&Rotace", rotation );
    m->insertItem("&Funkce", function );

    // Create a nice frame to put around the OpenGL widget
    QFrame* f = new QFrame( this, "frame" );
    f->setFrameStyle( QFrame::Sunken | QFrame::Panel );
    f->setLineWidth( 2 );

    // Create our OpenGL widget
    glWindow = new GLBox( f, "Open GL Widget", eval);

    // Create the three sliders; one for each rotation axis
    QSlider* x = new QSlider ( 0, 360, 60, 120, QSlider::Vertical, this, "xsl" );
    x->setTickmarks( QSlider::Left );
    QObject::connect( x, SIGNAL(valueChanged(int)),glWindow,SLOT(setXRotation(int)) );

    QSlider* y = new QSlider ( 0, 360, 60, 180, QSlider::Vertical, this, "ysl" );
    y->setTickmarks( QSlider::Left );
    QObject::connect( y, SIGNAL(valueChanged(int)),glWindow,SLOT(setYRotation(int)) );

    QSlider* z = new QSlider ( 0, 360, 60, 135, QSlider::Vertical, this, "zsl" );
    z->setTickmarks( QSlider::Left );
    QObject::connect( z, SIGNAL(valueChanged(int)),glWindow,SLOT(setZRotation(int)) );
	z->setValue(135);

    // Create move slider (Z axis)
    QSlider* zm = new QSlider ( -600, -60, 50, -200, QSlider::Vertical, this, "zmsl" );
    zm->setTickmarks( QSlider::Left );
    QObject::connect( zm, SIGNAL(valueChanged(int)),glWindow,SLOT(setZPosition(int)) );
	
	// Initialize GL scene position
	glWindow->setXRotation( x->value() );
	glWindow->setYRotation( y->value() );
	glWindow->setZRotation( z->value() );
	glWindow->setZPosition( zm->value() );


    // Now that we have all the widgets, put them into a nice layout

    // Put the sliders on top of each other
    QVBoxLayout* vlayout = new QVBoxLayout( 20, "vlayout");
    vlayout->addWidget( x );
    vlayout->addWidget( y );
    vlayout->addWidget( z );
    vlayout->addWidget( zm );

    // Put the GL widget inside the frame
    QHBoxLayout* flayout = new QHBoxLayout( f, 2, 2, "flayout");
    flayout->addWidget( glWindow, 1 );

    // Top level layout, puts the sliders to the left of the frame/GL widget
    QHBoxLayout* hlayout = new QHBoxLayout( this, 20, 20, "hlayout");
    hlayout->setMenuBar( m );
    hlayout->addLayout( vlayout );
    hlayout->addWidget( f, 1 );

	// sets default rotation factors
	glWindow->setXRotationFactor(2);
	glWindow->setYRotationFactor(2);
	glWindow->setZRotationFactor(2);

	// create instance of the timer
	timer = new QTimer( this );
    connect( timer, SIGNAL(timeout()), glWindow, SLOT(timerMove()) );
}

void GLObjectWindow::moveScene() {
}

void GLObjectWindow::menuRotateStart() {
	rotating = true;

	// Disable the sliders
    timer->start( 10 );
}

void GLObjectWindow::menuRotateStop() {
	rotating = false;
    timer->stop();
}

void GLObjectWindow::menuAbout() {
	QMessageBox::information( this, "O aplikaci", "Zapocet do geometrie\nLukas Zapletal\n\nProgram obsahuje interpret Perlu od firmy ActiveState a je postaven nad knihovnou QT od firmy Trolltech.");
}

void GLObjectWindow::menuFunctionDefine() {
	dlg->show();
	menuFunctionPlot();
}

void GLObjectWindow::menuFunctionPlot() {
	glWindow->RebuildScene();
}

void GLObjectWindow::menuRotateFactors() {
  bool ok;
  int res;

  ok = FALSE;
  res = QInputDialog::getInteger( "Vloz faktor rotace podle osy X", "Rotace", 2, 1, 360, 1, &ok, this );
  if ( ok )
      emit glWindow->setXRotationFactor(res);
  else
      return;

  ok = FALSE;
  res = QInputDialog::getInteger( "Vloz faktor rotace podle osy Y", "Rotace", 2, 1, 360, 1, &ok, this );
  if ( ok )
      emit glWindow->setYRotationFactor(res);
  else
      return;
  ok = FALSE;
  res = QInputDialog::getInteger( "Vloz faktor rotace podle osy Z", "Rotace", 2, 1, 360, 1, &ok, this );
  if ( ok )
      emit glWindow->setZRotationFactor(res);
  else
      return;
}

Evalutor* GLObjectWindow::getEvalutor()
{
	return eval;
}
